@@ -1,3 +1,22 @@
+/*
+ * Copyright (C) 2009 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>.
+ *
+ * This file is part of XChat-Guile.
+ *
+ * XChat-Guile is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * XChat-Guile is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ */
 #include "xchat-plugin.h"
 #include <libguile.h>
 #include <string.h>
@@ -10,9 +29,12 @@
 	
 #define PLUGIN_INFO	"Scheme (Guile) plugin for XChat - Version " \
 			PLUGIN_VERSION \
- 			"\nCopyright 2005 Zeeshan Ali"
+ 			"\nCopyright 2005-2008 Zeeshan Ali (Khattak) " \
+			"<zeeshanak@gnome.org>" \
+ 			"\nCopyright 2006 Lionel Elie Mamane <lionel@mamane.lu>"
 #define	PLUGIN_COMMAND	"guile"
 #define	PLUGIN_USAGE	"Usage: /guile eval SCHEME_EXPRESSION\n" \
+                        "              console\n" \
 			"              about"
 
 #define	TO_PRINT_MAX	10240
@@ -55,15 +77,13 @@ static SCM port;
 static xchat_hook *guile_cmd;
 static xchat_hook *load_cmd;
 static xchat_hook *unload_cmd;
+static xchat_hook *console_cmd;
 
 static scm_t_bits xchat_context_t;
 
 static gchar *to_print;
 
 static GSList *all_hooks = NULL;
-#ifdef USE_THREADS
-static SCM lock; /* one global lock for all global varriables */
-#endif
 
 static SCM
 xchat_context_equalp (SCM a, SCM b)
@@ -92,26 +112,6 @@ scm_body_thunk_2 (void *body_data)
     return scm_call_2 (c->body_proc, c->arg1, c->arg2);
 }
 
-static SCM
-scm_new_port_table_entry (scm_t_bits tag)
-#define FUNC_NAME "scm_new_port_table_entry"
-{
-    SCM port;
-    scm_t_port *pt;
-
-    SCM_NEWCELL (port);
-    pt = scm_add_to_port_table (port);
-    SCM_SET_CELL_TYPE (port, tag);
-    SCM_SETPTAB_ENTRY (port, pt);
-    return port;
-}
-#undef FUNC_NAME
-
-static inline int scm2bool (SCM obj)
-{
-    return (SCM_FALSEP (obj)) ? 0 : 1;
-}
-
 static gint
 xchat_fill_input (SCM port)
 {
@@ -141,12 +141,12 @@ xchat_write (SCM port, const void *data, size_t size)
 {
     gchar *str = g_strndup ((gchar *) data, size);
 
-#ifdef USE_THREADS
-    scm_lock_mutex (lock);
-#endif
-    if (to_print != NULL)
-	to_print = g_strjoin (NULL, to_print, str, NULL);
-
+    if (to_print != NULL) {
+	gchar *tmp;
+	tmp = g_strjoin (NULL, to_print, str, NULL);
+	g_free(to_print);
+	to_print = tmp;
+    }
     else
 	to_print = g_strdup (str);
 
@@ -154,9 +154,6 @@ xchat_write (SCM port, const void *data, size_t size)
 	    g_utf8_strlen (to_print, -1) >= TO_PRINT_MAX) {
 	xchat_flush_print ();
     }
-#ifdef USE_THREADS
-    scm_unlock_mutex (lock);
-#endif
 
     g_free (str);
 }
@@ -175,15 +172,21 @@ eval_func (gchar * expression)
     if (!SCM_EQ_P (value, SCM_UNDEFINED) && !SCM_EQ_P (value, SCM_UNSPECIFIED))
 	scm_display (value, port);
 
-#ifdef USE_THREADS
-    scm_lock_mutex (lock);
-#endif
     xchat_flush_print ();
-#ifdef USE_THREADS
-    scm_unlock_mutex (lock);
-#endif
 }
 
+/* LEM: Adapted from python plugin IInterp_Cmd */
+static int
+console_command(char *word[], char *word_eol[], void *userdata)
+{
+	char *channel = (char *) xchat_get_info(ph, "channel");
+	if (channel && channel[0] == '>' && strcmp(channel, ">>guile<<") == 0) {
+		xchat_printf(ph, ">>> %s\n", word_eol[1]);
+		eval_func(word_eol[1]);
+		return XCHAT_EAT_XCHAT;
+	}
+	return XCHAT_EAT_NONE;
+}
 static gboolean
 load_plugin (gchar * filename)
 {
@@ -251,6 +254,9 @@ guile_command (gchar * word[], gchar * word_eol[], void *userdata)
     else if (g_ascii_strcasecmp (word[2], "eval") == 0 && word_eol[3][0] != 0)
 	eval_func (word_eol[3]);
 
+    else if (g_ascii_strcasecmp (word[2], "console") == 0)
+      xchat_command(ph, "QUERY >>guile<<");
+
     else if (g_ascii_strcasecmp (word[2], "load") == 0) {
 	load_plugin (word[3]);
     }
@@ -313,7 +319,7 @@ unload_command (gchar * word[], gchar * word_eol[], void *userdata)
 static SCM 
 api_print (SCM str)
 {
-    xchat_printf (ph, "%s", SCM_STRING_CHARS (str));
+    xchat_printf (ph, "%s", scm_i_string_chars (str));
 
     return SCM_UNSPECIFIED;
 }
@@ -322,7 +328,7 @@ static SCM
 api_emit_print (SCM event_name, SCM args)
 {
     /* We'll only take the first 9 args passed to us */
-    gchar *argv[10];
+    const gchar *argv[10];
     gint result, i;
     SCM lst;
 
@@ -330,13 +336,13 @@ api_emit_print (SCM event_name, SCM args)
 
     lst = args;
 
-    for (i=0; i<9 && !scm2bool (scm_null_p (lst)); i++) {
-	argv[i] = SCM_STRING_CHARS (SCM_CAR (lst));
+    for (i=0; i<9 && !scm_to_bool (scm_null_p (lst)); i++) {
+	argv[i] = scm_i_string_chars (SCM_CAR (lst));
 	lst = SCM_CDR (lst);
     }
 
     result = xchat_emit_print (ph, 
-	    SCM_STRING_CHARS (event_name),
+	    scm_i_string_chars (event_name),
 	    argv[0],
 	    argv[1],
 	    argv[2],
@@ -353,7 +359,7 @@ api_emit_print (SCM event_name, SCM args)
 static SCM 
 api_command (SCM command)
 {
-    xchat_command (ph, SCM_STRING_CHARS (command));
+    xchat_command (ph, scm_i_string_chars (command));
 
     return SCM_UNSPECIFIED;
 }
@@ -371,17 +377,17 @@ static SCM
 api_find_context (SCM server, SCM channel)
 {
     xchat_context *context;
-    gchar *server_name, *channel_name;
+    const gchar *server_name, *channel_name;
 
     if (server == SCM_BOOL_F)
 	server_name = NULL;
     else
-	server_name = SCM_STRING_CHARS (server);
+	server_name = scm_i_string_chars (server);
 
     if (channel == SCM_BOOL_F)
 	channel_name = NULL;
     else
-	channel_name = SCM_STRING_CHARS (channel);
+	channel_name = scm_i_string_chars (channel);
 
     context = xchat_find_context (ph, server_name, channel_name);
 
@@ -422,8 +428,8 @@ api_nick_compare (SCM nick1, SCM nick2)
     int res;
 
     res = xchat_nickcmp (ph, 
-	    SCM_STRING_CHARS (nick1),
-	    SCM_STRING_CHARS (nick2));
+	    scm_i_string_chars (nick1),
+	    scm_i_string_chars (nick2));
 
     return scm_int2num (res);
 }
@@ -433,7 +439,7 @@ api_get_info (SCM id)
 {
     const gchar *info;
 
-    info = xchat_get_info (ph, SCM_STRING_CHARS (id));
+    info = xchat_get_info (ph, scm_i_string_chars (id));
 
     return scm_makfrom0str (info);
 }
@@ -452,16 +458,18 @@ api_send_modes (SCM targets, SCM modes_per_line, SCM sign, SCM mode)
     else {
 	gint i;
 	/* Allocate the array of char pointers */
-	gchar **targets_array = g_malloc (ntargets * sizeof (gchar *));
+	const gchar **targets_array = g_malloc (ntargets * sizeof (gchar *));
 
 	for (i=0; i<ntargets; i++)
 	    targets_array[i] = 
-		SCM_STRING_CHARS (scm_list_ref (targets, scm_int2num (i)));
+		scm_i_string_chars (scm_list_ref (targets, scm_int2num (i)));
 	xchat_send_modes (ph,
-		(const gchar **) targets_array,
-		ntargets, SCM_INUM (modes_per_line),
+		targets_array,
+		ntargets, scm_to_int (modes_per_line),
 		SCM_CHAR (sign),
 		SCM_CHAR (mode));
+
+        g_free (targets_array);
     }
 
     return SCM_UNSPECIFIED;
@@ -474,9 +482,9 @@ api_list_fields (SCM list_name)
     const gchar * const *fields;
     gint i;
 
-    scm_list = SCM_LIST0;
+    scm_list = SCM_EOL;
 
-    fields = xchat_list_fields (ph, SCM_STRING_CHARS (list_name));
+    fields = xchat_list_fields (ph, scm_i_string_chars (list_name));
     if (fields == NULL)
 	return SCM_BOOL_F;
 
@@ -497,19 +505,19 @@ api_list_get (SCM list_name)
     const gchar * const *fields;
     gint i;
 
-    list = xchat_list_get (ph, SCM_STRING_CHARS (list_name));
+    list = xchat_list_get (ph, scm_i_string_chars (list_name));
     if (list == NULL)
 	return SCM_BOOL_F;
 
-    fields = xchat_list_fields (ph, SCM_STRING_CHARS (list_name));
+    fields = xchat_list_fields (ph, scm_i_string_chars (list_name));
     if (fields == NULL) {
 	xchat_list_free(ph, list);
 	return SCM_BOOL_F;
     }
 
-    scm_list = SCM_LIST0;
+    scm_list = SCM_EOL;
     while (xchat_list_next (ph, list)) {
-	SCM inner_list = SCM_LIST0;
+	SCM assoc_list = SCM_EOL;
 	for (i = 0; fields[i]; i++) {
 	    const gchar *fld = fields[i]+1;
 	    SCM attr = NULL;
@@ -540,11 +548,13 @@ api_list_get (SCM list_name)
 		    attr = NULL;
 	    }
 
-	    if (attr != NULL)
-		inner_list = scm_cons (attr, inner_list);
+	    if (attr != NULL) {
+		SCM field_name = scm_makfrom0str (fields[i]+1);
+		assoc_list = scm_acons (field_name, attr, assoc_list);
+	    }
 	}
 
-	scm_list = scm_cons (scm_reverse (inner_list), scm_list);
+	scm_list = scm_cons (scm_reverse (assoc_list), scm_list);
     }
 
     xchat_list_free(ph, list);
@@ -554,7 +564,7 @@ api_list_get (SCM list_name)
 static SCM
 scm_list_from_strarray (const gchar **array)
 {
-    SCM list = SCM_LIST0;
+    SCM list = SCM_EOL;
     gint i;
 
     for (i = 0; array[i] && array[i][0]; i++)
@@ -636,10 +646,10 @@ api_hook_command (SCM command_name,
     scm_callback = scm_gc_protect_object (scm_callback);
 
     hook = xchat_hook_command (ph,
-	    SCM_STRING_CHARS (command_name),
+	    scm_i_string_chars (command_name),
 	    xchat_priority_symbol_to_int (priority),
 	    command_callback,
-	    SCM_STRING_CHARS (help_text),
+	    scm_i_string_chars (help_text),
 	    (void *)  SCM_UNPACK (scm_callback));
 
     if (hook == NULL) {
@@ -651,13 +661,7 @@ api_hook_command (SCM command_name,
 	/* According to Andreas Rottmann casting of (void *) to a long and
 	 * back is portable on all architectures. I hope he is correct.
 	 */
-#ifdef USE_THREADS
-	scm_lock_mutex (lock);
-#endif
 	all_hooks = g_slist_append (all_hooks, hook);
-#ifdef USE_THREADS
-	scm_unlock_mutex (lock);
-#endif
 	return scm_long2num ((glong) hook);
     }
 }
@@ -690,7 +694,7 @@ api_hook_print (SCM name,
     scm_callback = scm_gc_protect_object (scm_callback);
 
     hook = xchat_hook_print (ph,
-	    SCM_STRING_CHARS (name),
+	    scm_i_string_chars (name),
 	    xchat_priority_symbol_to_int (priority),
 	    print_callback,
 	    (void *)  SCM_UNPACK (scm_callback));
@@ -704,13 +708,7 @@ api_hook_print (SCM name,
 	/* According to Andreas Rottmann casting of (void *) to a long and
 	 * back is portable on all architectures. I hope he is correct.
 	 */
-#ifdef USE_THREADS
-	scm_lock_mutex (lock);
-#endif
 	all_hooks = g_slist_append (all_hooks, hook);
-#ifdef USE_THREADS
-	scm_unlock_mutex (lock);
-#endif
 	return scm_long2num ((glong) hook);
     }
 }
@@ -725,7 +723,7 @@ api_hook_server (SCM name,
     scm_callback = scm_gc_protect_object (scm_callback);
 
     hook = xchat_hook_server (ph,
-	    SCM_STRING_CHARS (name),
+	    scm_i_string_chars (name),
 	    xchat_priority_symbol_to_int (priority),
 	    command_callback,
 	    (void *)  SCM_UNPACK (scm_callback));
@@ -739,13 +737,7 @@ api_hook_server (SCM name,
 	/* According to Andreas Rottmann casting of (void *) to a long and
 	 * back is portable on all architectures. I hope he is correct.
 	 */
-#ifdef USE_THREADS
-	scm_lock_mutex (lock);
-#endif
 	all_hooks = g_slist_append (all_hooks, hook);
-#ifdef USE_THREADS
-	scm_unlock_mutex (lock);
-#endif
 	return scm_long2num ((glong) hook);
     }
 }
@@ -789,13 +781,7 @@ api_hook_timer (SCM timeout,
 	/* According to Andreas Rottmann casting of (void *) to a long and
 	 * back is portable on all architectures. I hope he is correct.
 	 */
-#ifdef USE_THREADS
-	scm_lock_mutex (lock);
-#endif
 	all_hooks = g_slist_append (all_hooks, hook);
-#ifdef USE_THREADS
-	scm_unlock_mutex (lock);
-#endif
 	return scm_long2num ((glong) hook);
     }
 }
@@ -818,13 +804,8 @@ api_unhook (SCM hook_id)
 {
     void *data = (void *) scm_num2long (hook_id, 0, NULL);
 
-#ifdef USE_THREADS
-    scm_lock_mutex (lock);
-#endif
     all_hooks = g_slist_remove (all_hooks, data);
-#ifdef USE_THREADS
-    scm_unlock_mutex (lock);
-#endif
+
     return real_unhook_func (data);
 }
 
@@ -838,10 +819,10 @@ SCM
 api_plugin_gui_add (SCM filename, SCM name, SCM description, SCM version)
 {
     void * handle = xchat_plugingui_add (ph,
-	    SCM_STRING_CHARS (filename),
-	    SCM_STRING_CHARS (name),
-	    SCM_STRING_CHARS (description),
-	    SCM_STRING_CHARS (version),
+	    scm_i_string_chars (filename),
+	    scm_i_string_chars (name),
+	    scm_i_string_chars (description),
+	    scm_i_string_chars (version),
 	    NULL);
 
     return scm_long2num ((glong) handle);
@@ -921,10 +902,6 @@ xchat_plugin_init (xchat_plugin * plugin_handle,
     /* initializing the guile library */
     scm_init_guile ();
 
-#ifdef USE_THREADS
-    lock = scm_gc_protect_object (scm_make_mutex ());
-#endif
-
     scm_c_define_module("xchat-guile main", module_init, NULL);
     scm_c_use_module ("xchat-guile main");
     scm_c_use_module ("xchat-guile plugin-system");
@@ -952,6 +929,13 @@ xchat_plugin_init (xchat_plugin * plugin_handle,
 	    NULL,
 	    NULL);
 
+    console_cmd = xchat_hook_command (ph,
+	    "",
+	    XCHAT_PRI_NORM,
+	    console_command,
+	    NULL,
+	    NULL);
+
     /* A 5 second commercial is all that is needed to be famous :) */
     xchat_print (ph, PLUGIN_INFO);
 
@@ -970,6 +954,7 @@ xchat_plugin_deinit (xchat_plugin * plugin_handle,
     xchat_unhook (ph, guile_cmd);
     xchat_unhook (ph, load_cmd);
     xchat_unhook (ph, unload_cmd);
+    xchat_unhook (ph, console_cmd);
 
     c.body_proc = SCM_VARIABLE_REF (scm_c_lookup ("xchat:unload-all-plugins"));
     c.tag = SCM_UNSPECIFIED;
@@ -980,20 +965,18 @@ xchat_plugin_deinit (xchat_plugin * plugin_handle,
 	    (scm_t_catch_handler) scm_handle_by_message_noexit,
 	    NULL);
 
-#ifdef USE_THREADS
-    scm_lock_mutex (lock);
-#endif
     g_slist_foreach (all_hooks, unhook_func, NULL);
     g_slist_free (all_hooks);
     
     if (to_print != NULL)
 	g_free (to_print);
 
-#ifdef USE_THREADS
-    scm_unlock_mutex (lock);
-    scm_gc_unprotect_object (lock);
-#endif
-
     return 1;
 }
 
+/* Settings for the c-mode of Emacs
+Local Variables:
+c-basic-offset: 4
+End:
+*/
+
